import json
import sys
import ethereum.testutils as testutils

from ethereum.slogging import get_logger, configure_logging
logger = get_logger()
# customize VM log output to your needs
# hint: use 'py.test' with the '-s' option to dump logs to the console
if '--trace' in sys.argv:  # not default
    configure_logging(':trace')
    sys.argv.remove('--trace')


def test_state(filename, testname, testdata):
    logger.debug('running test:%r in %r' % (testname, filename))
    testutils.check_state_test(testutils.fixture_to_bytes(testdata))


def pytest_generate_tests(metafunc):
    testutils.generate_test_params(
        'StateTests',
        metafunc,
        lambda filename, _, __: (
            'stQuadraticComplexityTest.json' in filename or
            'stMemoryStressTest.json' in filename or
            'stPreCompiledContractsTransaction.json' in filename
        )
    )


def main():
    global fixtures, filename, tests, testname, testdata
    if len(sys.argv) == 1:
        # read fixture from stdin
        fixtures = {'stdin': json.load(sys.stdin)}
    else:
        # load fixtures from specified file or dir
        try:
            fixtures = testutils.get_tests_from_file_or_dir(sys.argv[1])
        except:
            fixtures = {'stdin': json.loads(sys.argv[1])}
    for filename, tests in list(fixtures.items()):
        for testname, testdata in list(tests.items()):
            if len(sys.argv) < 3 or testname == sys.argv[2]:
                print("Testing: %s %s" % (filename, testname))
                testutils.check_state_test(testdata)


if __name__ == '__main__':
    main()



import rlp
from ethereum import blocks
from ethereum.blocks import Account, BlockHeader, Block, CachedBlock
from ethereum.utils import is_numeric, is_string, encode_hex, decode_hex, zpad, scan_bin, big_endian_to_int
from ethereum.securetrie import SecureTrie
from ethereum.trie import BLANK_NODE, BLANK_ROOT
from ethereum.pruning_trie import Trie


class FakeHeader(object):
    def __init__(self, number, hash, state_root, gas_limit, timestamp):
        self.number = number
        self.hash = hash
        self.state_root = state_root
        self.gas_limit = gas_limit
        self.timestamp = timestamp


class FakeBlock(object):
    def __init__(self, env, header, chain_diff):
        self.env = env
        self.config = env.config
        self.header = header
        self.uncles = []
        self.number = header.number
        self.hash = header.hash
        self.gas_limit = header.gas_limit
        self.difficulty = header.difficulty
        self.timestamp = header.timestamp
        self._chain_diff = chain_diff

    def chain_difficulty(self):
        return self._chain_diff

    def has_parent(self):
        return False

    def get_ancestor_list(self, n):
        if n == 0 or self.header.number == 0:
            return []
        p = FakeBlock(self.env, self.header, 0)
        return [p] + p.get_ancestor_list(n - 1)


def create_snapshot(chain, recent=1024):
    env = chain.env
    assert recent > env.config['MAX_UNCLE_DEPTH']+2

    head_block = chain.head
    base_block_hash = chain.index.get_block_by_number(max(head_block.number-recent, 0))
    base_block = chain.get(base_block_hash)

    snapshot = create_env_snapshot(base_block)
    snapshot['base'] = create_base_snapshot(base_block)
    snapshot['blocks'] = create_blocks_snapshot(base_block, head_block)
    snapshot['alloc'] = create_state_snapshot(env, base_block.state)

    return snapshot


def create_env_snapshot(base):
    return {
        'chainDifficulty': snapshot_form(base.chain_difficulty())
    }


def create_base_snapshot(base):
    return snapshot_form(rlp.encode(base.header))


def create_state_snapshot(env, state_trie):
    alloc = dict()
    count = 0
    for addr, account_rlp in state_trie.iter_branch():
        alloc[encode_hex(addr)] = create_account_snapshot(env, account_rlp)
        count += 1
        print "[%d] created account snapshot %s" % (count, encode_hex(addr))
    return alloc


def create_account_snapshot(env, rlpdata):
    account = get_account(env, rlpdata)
    storage_trie = SecureTrie(Trie(env.db, account.storage))
    storage = dict()
    for k, v in storage_trie.iter_branch():
        storage[encode_hex(k.lstrip('\x00') or '\x00')] = encode_hex(v)
    return {
        'nonce': snapshot_form(account.nonce),
        'balance': snapshot_form(account.balance),
        'code': encode_hex(account.code),
        'storage': storage
    }


def create_blocks_snapshot(base, head):
    recent_blocks = list()
    block = head
    while True:
        recent_blocks.append(snapshot_form(rlp.encode(block)))
        if block.prevhash != base.hash:
            block = block.get_parent()
        else:
            break
    recent_blocks.reverse()
    return recent_blocks


def load_snapshot(chain, snapshot):
    base_header = rlp.decode(scan_bin(snapshot['base']), BlockHeader)

    limit = len(snapshot['blocks'])
    # first block is child of base block
    first_block_rlp = scan_bin(snapshot['blocks'][0])
    first_header_data = rlp.decode(first_block_rlp)[0]
    head_block_rlp = scan_bin(snapshot['blocks'][limit-1])
    head_header_data = rlp.decode(head_block_rlp)[0]

    state = load_state(chain.env, snapshot['alloc'])
    assert state.root_hash == base_header.state_root

    _get_block_header = blocks.get_block_header
    def get_block_header(db, blockhash):
        if blockhash == first_header_data[0]:  # first block's prevhash
            return base_header
        return _get_block_header(db, blockhash)
    blocks.get_block_header = get_block_header

    _get_block = blocks.get_block
    def get_block(env, blockhash):
        if blockhash == first_header_data[0]:
            return FakeBlock(env, get_block_header(env.db, blockhash), int(snapshot['chainDifficulty']))
        return _get_block(env, blockhash)
    blocks.get_block = get_block

    def validate_uncles():
        return True

    print "Start loading recent blocks from snapshot"
    first_block = rlp.decode(first_block_rlp, Block, env=chain.env)
    chain.index.add_block(first_block)
    chain._store_block(first_block)
    chain.blockchain.put('HEAD', first_block.hash)
    chain.blockchain.put(chain.index._block_by_number_key(first_block.number), first_block.hash)
    chain.blockchain.commit()
    chain._update_head_candidate()

    count = 0
    for block_rlp in snapshot['blocks'][1:]:
        block_rlp = scan_bin(block_rlp)
        block = rlp.decode(block_rlp, Block, env=chain.env)
        if count < chain.env.config['MAX_UNCLE_DEPTH']+2:
            block.__setattr__('validate_uncles', validate_uncles)
        if not chain.add_block(block):
            print "Failed to load block #%d (%s), abort." % (block.number, encode_hex(block.hash)[:8])
        else:
            count += 1
            print "[%d] block #%d (%s) added" % (count, block.number, encode_hex(block.hash)[:8])
    print "Snapshot loaded."


def load_state(env, alloc):
    db = env.db
    state = SecureTrie(Trie(db, BLANK_ROOT))
    count = 0
    print "Start loading state from snapshot"
    for addr in alloc:
        account = alloc[addr]
        acct = Account.blank_account(db, env.config['ACCOUNT_INITIAL_NONCE'])
        if len(account['storage']) > 0:
            t = SecureTrie(Trie(db, BLANK_ROOT))
            for k in account['storage']:
                v = account['storage'][k]
                enckey = zpad(decode_hex(k), 32)
                t.update(enckey, decode_hex(v))
            acct.storage = t.root_hash
        if account['nonce']:
            acct.nonce = int(account['nonce'])
        if account['balance']:
            acct.balance = int(account['balance'])
        if account['code']:
            acct.code = decode_hex(account['code'])
        state.update(decode_hex(addr), rlp.encode(acct))
        count += 1
        if count % 1000 == 0:
            db.commit()
        print "[%d] loaded account %s" % (count, addr)
    db.commit()
    return state


def get_account(env, rlpdata):
    if rlpdata != BLANK_NODE:
        return rlp.decode(rlpdata, Account, db=env.db)
    else:
        return Account.blank_account(env.db, env.config['ACCOUNT_INITIAL_NONCE'])


def snapshot_form(val):
    if is_numeric(val):
        return str(val)
    elif is_string(val):
        return b'0x' + encode_hex(val)



import os
import ethereum.testutils as testutils
import ethereum.utils as utils
from ethereum.slogging import get_logger
import ethereum.keys as keys
logger = get_logger()


def test_key(filename, testname, testdata,):
    logger.debug('running test:%r in %r' % (testname, filename))
    assert keys.check_keystore_json(testdata["json"])
    privkey = keys.decode_keystore_json(testdata["json"], testdata["password"])
    assert utils.encode_hex(privkey) == utils.to_string(testdata["priv"])


def pytest_generate_tests(metafunc):
    testutils.generate_test_params('KeyStoreTests', metafunc)



from ethereum import tester as t
from ethereum.slogging import set_level
import time


serpent_code = """
def sqrt(n):
    i = 0
    while i * i < n:
        i += 1
    return i

def mul(a:arr, b:arr):
    c = array(len(a))
    sz = self.sqrt(len(a))
    i = 0
    while i < sz:
        j = 0
        while j < sz:
            k = 0
            while k < sz:
                c[i * sz + j] += a[i * sz + k] * b[k * sz + j]
                k += 1
            j += 1
        i += 1
    return(c:arr)

def exp(a:arr, i):
    if i == 1:
        return(a:arr)
    elif i % 2 == 0:
        m = self.exp(a, i / 2, outitems=len(a))
        return(self.mul(m, m, outitems=len(a)):arr)
    elif i % 2 == 1:
        m = self.exp(a, i / 2, outitems=len(a))
        return(self.mul(self.mul(m, m, outitems=len(a)), a, outitems=len(a)):arr)
"""


def test_mul():
    set_level(None, 'info')
    s = t.state()
    c = s.abi_contract(serpent_code)
    assert c.mul([1, 0, 0, 1], [2, 3, 4, 5]) == [2, 3, 4, 5]
    assert c.exp([0, 1, -1, 0], 24) == [1, 0, 0, 1]
    assert c.exp([0, -1, 1, 0], 39) == [0, 1, -1, 0]
    t.gas_limit = 100000000
    x = time.time()
    c.exp([i for i in range(81)], 31415)
    print('Exponentiation done in: %f' % (time.time() - x))

if __name__ == '__main__':
    test_mul()



# -*- coding: utf8 -*-
import pytest

from ethereum.utils import (
    big_endian_to_int, int_to_big_endian, decode_hex, encode_hex, rzpad, sha3, zpad,
)
from ethereum.abi import (
    _canonical_type, decode_abi, decode_single, decint, encode_abi,
    encode_single, event_id, normalize_name, method_id, ContractTranslator,
    EncodingError, ValueOutOfBounds,
)
import ethereum.testutils as testutils

# pylint: disable=invalid-name


def test_canonical_types():
    # https://github.com/ethereum/wiki/wiki/Ethereum-Contract-ABI#types
    assert _canonical_type('bool') == 'bool'
    assert _canonical_type('address') == 'address'

    assert _canonical_type('int') == 'int256'
    assert _canonical_type('uint') == 'uint256'
    assert _canonical_type('fixed') == 'fixed128x128'
    assert _canonical_type('ufixed') == 'ufixed128x128'

    assert _canonical_type('int[]') == 'int256[]'
    assert _canonical_type('uint[]') == 'uint256[]'
    assert _canonical_type('fixed[]') == 'fixed128x128[]'
    assert _canonical_type('ufixed[]') == 'ufixed128x128[]'

    assert _canonical_type('int[100]') == 'int256[100]'
    assert _canonical_type('uint[100]') == 'uint256[100]'
    assert _canonical_type('fixed[100]') == 'fixed128x128[100]'
    assert _canonical_type('ufixed[100]') == 'ufixed128x128[100]'


def test_function_selector():
    # https://github.com/ethereum/wiki/wiki/Ethereum-Contract-ABI#function-selector-and-argument-encoding
    baz_selector = big_endian_to_int(decode_hex('CDCD77C0'))
    bar_selector = big_endian_to_int(decode_hex('AB55044D'))
    sam_selector = big_endian_to_int(decode_hex('A5643BF2'))
    f_selector = big_endian_to_int(decode_hex('8BE65246'))

    assert big_endian_to_int(sha3('baz(uint32,bool)')[:4]) == baz_selector
    assert big_endian_to_int(sha3('bar(fixed128x128[2])')[:4]) == bar_selector
    assert big_endian_to_int(sha3('sam(bytes,bool,uint256[])')[:4]) == sam_selector
    assert big_endian_to_int(sha3('f(uint256,uint32[],bytes10,bytes)')[:4]) == f_selector

    assert method_id('baz', ['uint32', 'bool']) == baz_selector
    assert method_id('bar', ['fixed128x128[2]']) == bar_selector
    assert method_id('sam', ['bytes', 'bool', 'uint256[]']) == sam_selector
    assert method_id('f', ['uint256', 'uint32[]', 'bytes10', 'bytes']) == f_selector

    assert method_id('bar', ['fixed[2]']) == bar_selector
    assert method_id('sam', ['bytes', 'bool', 'uint[]']) == sam_selector
    assert method_id('f', ['uint', 'uint32[]', 'bytes10', 'bytes']) == f_selector


def test_event():
    event_abi = [{
        'name': 'Test',
        'anonymous': False,
        'inputs': [
            {'indexed': False, 'name': 'a', 'type': 'int256'},
            {'indexed': False, 'name': 'b', 'type': 'int256'},
        ],
        'type': 'event',
    }]

    contract_abi = ContractTranslator(event_abi)

    normalized_name = normalize_name('Test')
    encode_types = ['int256', 'int256']
    id_ = event_id(normalized_name, encode_types)

    topics = [id_]
    data = encode_abi(encode_types, [1, 2])

    result = contract_abi.decode_event(topics, data)

    assert result['_event_type'] == b'Test'
    assert result['a'] == 1
    assert result['b'] == 2


def test_decint():
    assert decint(True) == 1
    assert decint(False) == 0
    assert decint(None) == 0

    # unsigned upper boundary
    assert decint(2 ** 256 - 1, signed=False) == 2 ** 256 - 1
    assert decint(int_to_big_endian(2 ** 256 - 1), signed=False) == 2 ** 256 - 1

    with pytest.raises(EncodingError):
        decint(2 ** 256, signed=False)
        decint(int_to_big_endian(2 ** 256), signed=False)

    # unsigned lower boundary
    assert decint(0) == 0
    assert decint(int_to_big_endian(0)) == 0
    with pytest.raises(EncodingError):
        decint(-1, signed=False)
        decint(int_to_big_endian(-1), signed=False)

    # signed upper boundary
    assert decint(2 ** 255 - 1, signed=True) == 2 ** 255 - 1
    assert decint(int_to_big_endian(2 ** 255 - 1), signed=True) == 2 ** 255 - 1

    with pytest.raises(EncodingError):
        decint(2 ** 255, signed=True)
        decint(int_to_big_endian(2 ** 255), signed=True)

    # signed lower boundary
    assert decint(-2 ** 255, signed=True) == -2 ** 255
    # assert decint(int_to_big_endian(-2 ** 255), signed=True) == -2 ** 255
    with pytest.raises(EncodingError):
        decint(-2 ** 255 - 1, signed=True)
        # decint(int_to_big_endian(-2 ** 255 - 1), signed=True)


def test_encode_int():
    int8 = ('int', '8', [])
    int32 = ('int', '32', [])
    int256 = ('int', '256', [])

    int256_maximum = (
        b'\x7f\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff'
        b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff'
    )
    int256_minimum = (
        b'\x80\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
        b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
    )
    int256_128 = (
        b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff'
        b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\x80'
    )
    int256_2_to_31 = (
        b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff'
        b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\x80\x00\x00\x00'
    )
    int256_negative_one = (
        b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff'
        b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff'
    )

    assert encode_single(int256, int256_minimum) == int256_minimum

    assert encode_single(int8, 0) == zpad(b'\x00', 32)
    assert encode_single(int8, 2 ** 7 - 1) == zpad(b'\x7f', 32)
    assert encode_single(int8, -1) == zpad(b'\xff', 32)
    assert encode_single(int8, -2 ** 7) == zpad(b'\x80', 32)

    with pytest.raises(ValueOutOfBounds):
        encode_single(int8, 128)

    with pytest.raises(ValueOutOfBounds):
        encode_single(int8, -129)

    assert encode_single(int32, 0) == zpad(b'\x00', 32)
    assert encode_single(int32, 2 ** 7 - 1) == zpad(b'\x7f', 32)
    assert encode_single(int32, 2 ** 31 - 1) == zpad(b'\x7f\xff\xff\xff', 32)
    assert encode_single(int32, -1) == zpad(b'\xff\xff\xff\xff', 32)
    assert encode_single(int32, -2 ** 7) == zpad(b'\xff\xff\xff\x80', 32)
    assert encode_single(int32, -2 ** 31) == zpad(b'\x80\x00\x00\x00', 32)

    with pytest.raises(ValueOutOfBounds):
        encode_single(int32, 2 ** 32)

    with pytest.raises(ValueOutOfBounds):
        encode_single(int32, -(2 ** 32))

    assert encode_single(int256, 0) == zpad(b'\x00', 32)
    assert encode_single(int256, 2 ** 7 - 1) == zpad(b'\x7f', 32)
    assert encode_single(int256, 2 ** 31 - 1) == zpad(b'\x7f\xff\xff\xff', 32)
    assert encode_single(int256, 2 ** 255 - 1) == int256_maximum
    assert encode_single(int256, -1) == int256_negative_one
    assert encode_single(int256, -2 ** 7) == int256_128
    assert encode_single(int256, -2 ** 31) == int256_2_to_31
    assert encode_single(int256, -2 ** 255) == int256_minimum

    with pytest.raises(ValueOutOfBounds):
        encode_single(int256, 2 ** 256)

    with pytest.raises(ValueOutOfBounds):
        encode_single(int256, -(2 ** 256))


def test_encode_uint():
    uint8 = ('uint', '8', [])
    uint32 = ('uint', '32', [])
    uint256 = ('uint', '256', [])

    with pytest.raises(ValueOutOfBounds):
        encode_single(uint8, -1)

    with pytest.raises(ValueOutOfBounds):
        encode_single(uint32, -1)

    with pytest.raises(ValueOutOfBounds):
        encode_single(uint256, -1)

    assert encode_single(uint8, 0) == zpad(b'\x00', 32)
    assert encode_single(uint32, 0) == zpad(b'\x00', 32)
    assert encode_single(uint256, 0) == zpad(b'\x00', 32)

    assert encode_single(uint8, 1) == zpad(b'\x01', 32)
    assert encode_single(uint32, 1) == zpad(b'\x01', 32)
    assert encode_single(uint256, 1) == zpad(b'\x01', 32)

    assert encode_single(uint8, 2 ** 8 - 1) == zpad(b'\xff', 32)
    assert encode_single(uint32, 2 ** 8 - 1) == zpad(b'\xff', 32)
    assert encode_single(uint256, 2 ** 8 - 1) == zpad(b'\xff', 32)

    assert encode_single(uint32, 2 ** 32 - 1) == zpad(b'\xff\xff\xff\xff', 32)
    assert encode_single(uint256, 2 ** 32 - 1) == zpad(b'\xff\xff\xff\xff', 32)

    uint256_maximum = (
        b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff'
        b'\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff'
    )
    assert encode_single(uint256, 2 ** 256 - 1) == uint256_maximum


def test_encode_bool():
    bool_ = ('bool', '', [])
    uint8 = ('uint', '8', [])

    assert encode_single(bool_, True) == zpad(b'\x01', 32)
    assert encode_single(bool_, False) == zpad(b'\x00', 32)

    assert encode_single(bool_, True) == encode_single(uint8, 1)
    assert encode_single(bool_, False) == encode_single(uint8, 0)


def test_encode_fixed():
    fixed128x128 = ('fixed', '128x128', [])

    _2_125 = decode_hex(
        b'00000000000000000000000000000002'
        b'20000000000000000000000000000000'
    )

    _8_5 = decode_hex(
        b'00000000000000000000000000000008'
        b'80000000000000000000000000000000'
    )

    assert encode_single(fixed128x128, 2.125) == _2_125
    assert encode_single(fixed128x128, 8.5) == _8_5

    assert encode_single(fixed128x128, 1.125) == (b'\x00' * 15 + b'\x01\x20' + b'\x00' * 15)
    assert encode_single(fixed128x128, -1.125) == (b'\xff' * 15 + b'\xfe' + b'\xe0' + b'\x00' * 15)

    with pytest.raises(ValueOutOfBounds):
        encode_single(fixed128x128, 2 ** 127)

    with pytest.raises(ValueOutOfBounds):
        encode_single(fixed128x128, -2 ** 127 - 1)


def test_encoded_ufixed():
    ufixed128x128 = ('ufixed', '128x128', [])

    _2_125 = decode_hex(
        b'00000000000000000000000000000002'
        b'20000000000000000000000000000000'
    )

    _8_5 = decode_hex(
        b'00000000000000000000000000000008'
        b'80000000000000000000000000000000'
    )

    assert encode_single(ufixed128x128, 2.125) == _2_125
    assert encode_single(ufixed128x128, 8.5) == _8_5

    assert encode_single(ufixed128x128, 0) == (b'\x00' * 32)
    assert encode_single(ufixed128x128, 1.125) == (b'\x00' * 15 + b'\x01\x20' + b'\x00' * 15)
    assert encode_single(ufixed128x128, 2**127 - 1) == (b'\x7f' + b'\xff' * 15 + b'\x00' * 16)

    with pytest.raises(ValueOutOfBounds):
        encode_single(ufixed128x128, 2 ** 128 + 1)

    with pytest.raises(ValueOutOfBounds):
        encode_single(ufixed128x128, -1)


def test_encode_dynamic_bytes():
    dynamic_bytes = ('bytes', '', [])
    uint256 = ('uint', '256', [])

    # only the size of the bytes
    assert encode_single(dynamic_bytes, b'') == zpad(b'\x00', 32)

    a = encode_single(uint256, 1) + rzpad(b'\x61', 32)
    assert encode_single(dynamic_bytes, b'\x61') == a

    dave_bin = decode_hex(
        b'0000000000000000000000000000000000000000000000000000000000000004'
        b'6461766500000000000000000000000000000000000000000000000000000000'
    )
    dave = encode_single(uint256, 4) + rzpad(b'\x64\x61\x76\x65', 32)
    assert encode_single(dynamic_bytes, b'\x64\x61\x76\x65') == dave_bin
    assert encode_single(dynamic_bytes, b'\x64\x61\x76\x65') == dave


def test_encode_dynamic_string():
    string = ('string', '', [])
    uint256 = ('uint', '256', [])

    a = u'ã'
    a_utf8 = a.encode('utf8')

    with pytest.raises(ValueError):
        encode_single(string, a.encode('latin'))

    a_encoded = encode_single(uint256, len(a_utf8)) + rzpad(a_utf8, 32)
    assert encode_single(string, a.encode('utf8')) == a_encoded


def test_encode_hash():
    hash8 = ('hash', '8', [])

    assert encode_single(hash8, b'\x00' * 8) == b'\x00' * 32
    assert encode_single(hash8, b'00' * 8) == b'\x00' * 32


def test_encode_address():
    prefixed_address = '0x' + '0' * 40
    assert encode_single(['address', '', []], prefixed_address) == b'\x00' * 32


def test_encode_decode_int():
    int8 = ('int', '8', [])
    int32 = ('int', '32', [])
    int256 = ('int', '256', [])

    int8_values = [
        1, -1,
        127, -128,
    ]
    int32_values = [
        1, -1,
        127, -128,
        2 ** 31 - 1, -2 ** 31,
    ]
    int256_values = [
        1, -1,
        127, -128,
        2 ** 31 - 1, -2 ** 31,
        2 ** 255 - 1, -2 ** 255,
    ]

    for value in int8_values:
        assert encode_abi(['int8'], [value]) == encode_single(int8, value)
        assert decode_abi(['int8'], encode_abi(['int8'], [value]))[0] == value

    for value in int32_values:
        assert encode_abi(['int32'], [value]) == encode_single(int32, value)
        assert decode_abi(['int32'], encode_abi(['int32'], [value]))[0] == value

    for value in int256_values:
        assert encode_abi(['int256'], [value]) == encode_single(int256, value)
        assert decode_abi(['int256'], encode_abi(['int256'], [value]))[0] == value


def test_encode_decode_bool():
    assert decode_abi(['bool'], encode_abi(['bool'], [True]))[0] is True
    assert decode_abi(['bool'], encode_abi(['bool'], [False]))[0] is False


def test_encode_decode_fixed():
    fixed128x128 = ('fixed', '128x128', [])

    fixed_data = encode_single(fixed128x128, 1)
    assert decode_single(fixed128x128, fixed_data) == 1

    fixed_data = encode_single(fixed128x128, 2**127 - 1)
    assert decode_single(fixed128x128, fixed_data) == (2**127 - 1) * 1.0

    fixed_data = encode_single(fixed128x128, -1)
    assert decode_single(fixed128x128, fixed_data) == -1

    fixed_data = encode_single(fixed128x128, -2**127)
    assert decode_single(fixed128x128, fixed_data) == -2**127


def test_encode_decode_bytes():
    bytes8 = ('bytes', '8', [])
    dynamic_bytes = ('bytes', '', [])

    assert decode_single(bytes8, encode_single(bytes8, b'\x01\x02')) == (b'\x01\x02' + b'\x00' * 6)
    assert decode_single(dynamic_bytes, encode_single(dynamic_bytes, b'\x01\x02')) == b'\x01\x02'


def test_encode_decode_hash():
    hash8 = ('hash', '8', [])

    hash1 = b'\x01' * 8
    assert hash1 == decode_single(hash8, encode_single(hash8, hash1))


def test_encode_decode_address():
    address1 = b'\x11' * 20
    address2 = b'\x22' * 20
    address3 = b'\x33' * 20

    all_addresses = [
        address1,
        address2,
        address3,
    ]
    all_addresses_encoded = [
        encode_hex(address1),
        encode_hex(address2),
        encode_hex(address3),
    ]

    assert decode_abi(['address'], encode_abi(['address'], [address1]))[0] == encode_hex(address1)

    addresses_encoded_together = encode_abi(['address[]'], [all_addresses])
    assert decode_abi(['address[]'], addresses_encoded_together)[0] == all_addresses_encoded

    address_abi = ['address', 'address', 'address']
    addreses_encoded_splited = encode_abi(address_abi, all_addresses)
    assert decode_abi(address_abi, addreses_encoded_splited) == all_addresses_encoded


def test_state(filename, testname, testdata):  # pylint: disable=unused-argument
    # test data generated by testutils.generate_test_params
    testutils.check_abi_test(testutils.fixture_to_bytes(testdata))


def pytest_generate_tests(metafunc):
    testutils.generate_test_params('ABITests', metafunc)



from ethereum.utils import mk_contract_address, encode_hex
source = '0x4a574510c7014e4ae985403536074abe582adfc8'

L = []
# DAO
L.append('0xbb9bc244d798123fde783fcc1c72d3bb8c189413')
# DAO extrabalance
L.append('0x807640a13483f8ac783c557fcdf27be11ea4ac7a')
# child DAOs (created by DAO creator)
L.extend([b'0x' + encode_hex(mk_contract_address(source, i)) for i in range(1, 58)])
# child extrabalances
L.extend([b'0x' + encode_hex(mk_contract_address(mk_contract_address(source, i), 0)) for i in range(1, 58)])



# -*- coding: utf8 -*-
import os
import re
import subprocess
import warnings
import shlex

import yaml

from rlp.utils import decode_hex
from . import utils

BINARY = 'solc'


class CompileError(Exception):
    pass


class SolcMissing(Exception):
    pass


def get_compiler_path():
    """ Return the path to the solc compiler.

    This funtion will search for the solc binary in the $PATH and return the
    path of the first executable occurence.
    """
    # If the user provides a specific solc binary let's use that
    given_binary = os.environ.get('SOLC_BINARY')
    if given_binary:
        return given_binary

    for path in os.getenv('PATH', '').split(os.pathsep):
        path = path.strip('"')
        executable_path = os.path.join(path, BINARY)

        if os.path.isfile(executable_path) and os.access(executable_path, os.X_OK):
            return executable_path

    return None


def get_solidity():
    """ Return the singleton used to interact with the solc compiler. """
    if get_compiler_path() is None:
        return None  # the compiler wasn't found in $PATH

    return solc_wrapper


def solc_arguments(libraries=None, combined='bin,abi', optimize=True, extra_args=None):
    """ Build the arguments to call the solc binary. """
    args = [
        '--combined-json', combined,
        '--add-std'
    ]

    if optimize:
        args.append('--optimize')

    if extra_args:
        try:
            args.extend(shlex.split(extra_args))
        except:  # if not a parseable string then treat it as a list
            args.extend(extra_args)

    if libraries is not None and len(libraries):
        addresses = [
            '{name}:{address}'.format(name=name, address=address.decode('utf8'))
            for name, address in libraries.items()
        ]
        args.extend([
            '--libraries',
            ','.join(addresses),
        ])

    return args


def solc_parse_output(compiler_output):
    """ Parses the compiler output. """
    # At the moment some solc output like --hashes or -- gas will not output
    # json at all so if used with those arguments the logic here will break.
    # Perhaps solidity will slowly switch to a json only output and this comment
    # can eventually go away and we will not need to add more logic here at all.
    result = yaml.safe_load(compiler_output)['contracts']

    if 'bin' in tuple(result.values())[0]:
        for value in result.values():
            value['bin_hex'] = value['bin']

            # decoding can fail if the compiled contract has unresolved symbols
            try:
                value['bin'] = decode_hex(value['bin_hex'])
            except TypeError:
                pass

    for json_data in ('abi', 'devdoc', 'userdoc'):
        # the values in the output can be configured through the
        # --combined-json flag, check that it's present in the first value and
        # assume all values are consistent
        if json_data not in tuple(result.values())[0]:
            continue

        for value in result.values():
            value[json_data] = yaml.safe_load(value[json_data])

    return result


def compiler_version():
    """ Return the version of the installed solc. """
    version_info = subprocess.check_output(['solc', '--version'])
    match = re.search(b'^Version: ([0-9a-z.-]+)/', version_info, re.MULTILINE)

    if match:
        return match.group(1)


def solidity_names(code):  # pylint: disable=too-many-branches
    """ Return the library and contract names in order of appearence. """
    names = []
    in_string = None
    backslash = False
    comment = None

    # "parse" the code by hand to handle the corner cases:
    #  - the contract or library can be inside a comment or string
    #  - multiline comments
    #  - the contract and library keywords could not be at the start of the line
    for pos, char in enumerate(code):
        if in_string:
            if not backslash and in_string == char:
                in_string = None
                backslash = False

            if char == '\\':  # pylint: disable=simplifiable-if-statement
                backslash = True
            else:
                backslash = False

        elif comment == '//':
            if char in ('\n', '\r'):
                comment = None

        elif comment == '/*':
            if char == '*' and code[pos + 1] == '/':
                comment = None

        else:
            if char == '"' or char == "'":
                in_string = char

            if char == '/':
                if code[pos + 1] == '/':
                    comment = '//'
                if code[pos + 1] == '*':
                    comment = '/*'

            if char == 'c' and code[pos: pos + 8] == 'contract':
                result = re.match('^contract[^_$a-zA-Z]+([_$a-zA-Z][_$a-zA-Z0-9]*)', code[pos:])

                if result:
                    names.append(('contract', result.groups()[0]))

            if char == 'l' and code[pos: pos + 7] == 'library':
                result = re.match('^library[^_$a-zA-Z]+([_$a-zA-Z][_$a-zA-Z0-9]*)', code[pos:])

                if result:
                    names.append(('library', result.groups()[0]))

    return names


def solidity_library_symbol(library_name):
    """ Return the symbol used in the bytecode to represent the `library_name`. """
    # the symbol is always 40 characters in length with the minimum of two
    # leading and trailing underscores
    length = min(len(library_name), 36)

    library_piece = library_name[:length]
    hold_piece = '_' * (36 - length)

    return '__{library}{hold}__'.format(
        library=library_piece,
        hold=hold_piece,
    )


def solidity_resolve_address(hex_code, library_symbol, library_address):
    """ Change the bytecode to use the given library address.

    Args:
        hex_code (bin): The bytecode encoded in hexadecimal.
        library_name (str): The library that will be resolved.
        library_address (str): The address of the library.

    Returns:
        bin: The bytecode encoded in hexadecimal with the library references
            resolved.
    """
    if library_address.startswith('0x'):
        raise ValueError('Address should not contain the 0x prefix')

    try:
        decode_hex(library_address)
    except TypeError:
        raise ValueError('library_address contains invalid characters, it must be hex encoded.')

    if len(library_symbol) != 40 or len(library_address) != 40:
        raise ValueError('Address with wrong length')

    return hex_code.replace(library_symbol, library_address)


def solidity_resolve_symbols(hex_code, libraries):
    symbol_address = {
        solidity_library_symbol(library_name): address
        for library_name, address in libraries.items()
    }

    for unresolved in solidity_unresolved_symbols(hex_code):
        address = symbol_address[unresolved]
        hex_code = solidity_resolve_address(hex_code, unresolved, address)

    return hex_code


def solidity_unresolved_symbols(hex_code):
    """ Return the unresolved symbols contained in the `hex_code`.

    Note:
        The binary representation should not be provided since this function
        relies on the fact that the '_' is invalid in hex encoding.

    Args:
        hex_code (str): The bytecode encoded as hexadecimal.
    """
    return set(re.findall(r"_.{39}", hex_code))


def compile_file(filepath, libraries=None, combined='bin,abi', optimize=True, extra_args=None):
    """ Return the compile contract code.

    Args:
        filepath (str): The path to the contract source code.
        libraries (dict): A dictionary mapping library name to it's address.
        combined (str): The argument for solc's --combined-json.
        optimize (bool): Enable/disables compiler optimization.

    Returns:
        dict: A mapping from the contract name to it's binary.
    """

    workdir, filename = os.path.split(filepath)

    args = solc_arguments(libraries=libraries, combined=combined, optimize=optimize, extra_args=extra_args)
    args.insert(0, get_compiler_path())
    args.append(filename)

    output = subprocess.check_output(args, cwd=workdir)

    return solc_parse_output(output)


def solidity_get_contract_data(all_contracts, filepath, contract_name):
    """ A backwards compatible method of getting the contract data out
    of a solc --combined-json output"""
    try:
        contract_data = all_contracts[contract_name]
    except:
        if filepath is None:
            filename = '<stdin>'
        else:
            _, filename = os.path.split(filepath)
        contract_data = all_contracts[filename + ":" + contract_name]
    return contract_data


def solidity_get_contract_key(all_contracts, filepath, contract_name):
    """ A backwards compatible method of getting the key to the all_contracts
    dictionary for a particular contract"""
    if contract_name in all_contracts:
        return contract_name
    else:
        if filepath is None:
            filename = '<stdin>'
        else:
            _, filename = os.path.split(filepath)
        contract_key = filename + ":" + contract_name
        return contract_key if contract_key in all_contracts else None


def compile_contract(filepath, contract_name, libraries=None, combined='bin,abi', optimize=True, extra_args=None):
    all_contracts = compile_file(
        filepath,
        libraries=libraries,
        combined=combined,
        optimize=optimize,
        extra_args=extra_args
    )
    return solidity_get_contract_data(all_contracts, filepath, contract_name)


def compile_last_contract(filepath, libraries=None, combined='bin,abi', optimize=True, extra_args=None):
    with open(filepath) as handler:
        all_names = solidity_names(handler.read())

    all_contract_names = [
        name
        for _, name in all_names
    ]

    last_contract = all_contract_names[-1]

    return compile_contract(
        filepath,
        last_contract,
        libraries=libraries,
        combined=combined,
        optimize=optimize,
        extra_args=extra_args
    )


def compile_code(sourcecode, libraries=None, combined='bin,abi', optimize=True, extra_args=None):
    args = solc_arguments(libraries=libraries, combined=combined, optimize=optimize, extra_args=extra_args)
    compiler = get_compiler_path()
    if compiler is None:
        raise SolcMissing("solc not found")
    args.insert(0, compiler)

    process = subprocess.Popen(args, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    stdoutdata, stderrdata = process.communicate(input=utils.to_string(sourcecode))

    if process.returncode != 0:
        raise CompileError(stderrdata)

    return solc_parse_output(stdoutdata)


class Solc(object):
    """ Wraps the solc binary. """

    compiler_available = staticmethod(get_compiler_path)
    contract_names = staticmethod(solidity_names)
    compiler_version = staticmethod(compiler_version)

    @staticmethod
    def _code_or_path(sourcecode, path, contract_name, libraries, combined, extra_args):
        warnings.warn('solc_wrapper is deprecated, please use the functions compile_file or compile_code')

        if sourcecode and path:
            raise ValueError('sourcecode and path are mutually exclusive.')

        if path and contract_name:
            return compile_contract(path, contract_name, libraries=libraries, combined=combined, extra_args=extra_args)

        if path:
            return compile_last_contract(path, libraries=libraries, combined=combined, extra_args=extra_args)

        all_names = solidity_names(sourcecode)
        all_contract_names = [
            name
            for _, name in all_names
        ]
        last_contract = all_contract_names[-1]

        result = compile_code(sourcecode, libraries=libraries, combined=combined, extra_args=extra_args)
        return solidity_get_contract_data(result, path, last_contract)

    @classmethod
    def compile(cls, code, path=None, libraries=None, contract_name='', extra_args=None):
        """ Return the binary of last contract in code. """
        result = cls._code_or_path(code, path, contract_name, libraries, 'bin', extra_args)
        return result['bin']

    @classmethod
    def mk_full_signature(cls, code, path=None, libraries=None, contract_name='', extra_args=None):
        "returns signature of last contract in code"

        result = cls._code_or_path(code, path, contract_name, libraries, 'abi', extra_args)
        return result['abi']

    @classmethod
    def combined(cls, code, path=None, extra_args=None):
        """ Compile combined-json with abi,bin,devdoc,userdoc.

        @param code: literal solidity code as a string.
        @param path: absolute path to solidity-file. Note: code & path are
                     mutually exclusive!
        @param extra_args: Either a space separated string or a list of extra
                           arguments to be passed to the solidity compiler.
        """

        if code and path:
            raise ValueError('sourcecode and path are mutually exclusive.')

        if path:
            contracts = compile_file(path, extra_args=extra_args)

            with open(path) as handler:
                code = handler.read()

        elif code:
            contracts = compile_code(code, extra_args=extra_args)

        else:
            raise ValueError('either code or path needs to be supplied.')

        sorted_contracts = []
        for name in solidity_names(code):
            sorted_contracts.append(
                (
                    name[1],
                    solidity_get_contract_data(contracts, path, name[1])
                )
            )
        return sorted_contracts

    @classmethod
    def compile_rich(cls, code, path=None, extra_args=None):
        """full format as returned by jsonrpc"""

        return {
            contract_name: {
                'code': '0x' + contract.get('bin_hex'),
                'info': {
                    'abiDefinition': contract.get('abi'),
                    'compilerVersion': cls.compiler_version(),
                    'developerDoc': contract.get('devdoc'),
                    'language': 'Solidity',
                    'languageVersion': '0',
                    'source': code,
                    'userDoc': contract.get('userdoc')
                },
            }
            for contract_name, contract
            in cls.combined(code, path=path, extra_args=extra_args)
        }


solc_wrapper = Solc  # pylint: disable=invalid-name



# -*- coding: utf8 -*-
import shutil
import tempfile
import time
import types
import warnings

import rlp
from rlp.utils import ascii_chr

from ethereum import blocks, db, opcodes, processblock, transactions
from ethereum.abi import ContractTranslator
from ethereum.config import Env
from ethereum.slogging import LogRecorder
from ethereum._solidity import get_solidity
from ethereum.utils import to_string, sha3, privtoaddr, int_to_addr

TRACE_LVL_MAP = [
    ':info',
    'eth.vm.log:trace',
    ':info,eth.vm.log:trace,eth.vm.exit:trace',
    ':info,eth.vm.log:trace,eth.vm.op:trace,eth.vm.stack:trace',
    ':info,eth.vm.log:trace,eth.vm.op:trace,eth.vm.stack:trace,' +
    'eth.vm.storage:trace,eth.vm.memory:trace'
]

GAS_LIMIT = 3141592
GAS_PRICE = 1

# pylint: disable=invalid-name

gas_limit = GAS_LIMIT
gas_price = GAS_PRICE

accounts = []
keys = []
languages = {}

for account_number in range(10):
    keys.append(sha3(to_string(account_number)))
    accounts.append(privtoaddr(keys[-1]))

k0, k1, k2, k3, k4, k5, k6, k7, k8, k9 = keys[:10]
a0, a1, a2, a3, a4, a5, a6, a7, a8, a9 = accounts[:10]

try:
    import serpent  # pylint: disable=wrong-import-position
    languages['serpent'] = serpent
except ImportError:
    pass

_solidity = get_solidity()
if _solidity:
    languages['solidity'] = _solidity

DEFAULT_KEY = k0
DEFAULT_ACCOUNT = a0


def dict_without(base_dict, *args):
    """ Return a copy of the dictionary without the `keys`. """
    without_keys = dict(base_dict)

    for key in args:
        without_keys.pop(key, None)

    return without_keys


def dict_with(base_dict, **kwargs):
    """ Return a copy of the dictionary with the added elements. """
    full_dict = dict(base_dict)
    full_dict.update(**kwargs)

    return full_dict


def rand():
    """ Return a pseudo random number.

    Pseudo-RNG (deterministic for now for testing purposes).
    """
    rand.seed = pow(rand.seed, 2, 2 ** 512)
    return rand.seed % 2 ** 256


# initialize the rand function
rand.seed = 3 ** 160


class TransactionFailed(Exception):
    pass


class ContractCreationFailed(Exception):
    pass


class ABIContract(object):  # pylint: disable=too-few-public-methods

    def __init__(self, _state, _abi, address, listen=True,  # pylint: disable=too-many-arguments
                 log_listener=None, default_key=None):
        self.address = address
        self.default_key = default_key or DEFAULT_KEY

        if isinstance(_abi, ContractTranslator):
            abi_translator = _abi
        else:
            abi_translator = ContractTranslator(_abi)

        self.translator = abi_translator

        def listener(log):
            result = self.translator.listen(log, noprint=False)

            if result and log_listener:
                log_listener(result)

        if listen:
            _state.block.log_listeners.append(listener)

        for function_name in self.translator.function_data:
            function = self.method_factory(_state, function_name)
            method = types.MethodType(function, self)
            setattr(self, function_name, method)

    @staticmethod
    def method_factory(test_state, function_name):
        """ Return a proxy for calling a contract method with automatic encoding of
        argument and decoding of results.
        """

        def kall(self, *args, **kwargs):
            key = kwargs.get('sender', self.default_key)

            result = test_state._send(  # pylint: disable=protected-access
                key,
                self.address,
                kwargs.get('value', 0),
                self.translator.encode(function_name, args),
                **dict_without(kwargs, 'sender', 'value', 'output')
            )

            if kwargs.get('output', '') == 'raw':
                outdata = result['output']
            elif not result['output']:
                outdata = None
            else:
                outdata = self.translator.decode(function_name, result['output'])
                outdata = outdata[0] if len(outdata) == 1 else outdata

            if kwargs.get('profiling', ''):
                return dict_with(result, output=outdata)

            return outdata
        return kall


class state(object):

    def __init__(self, num_accounts=len(keys)):
        self.temp_data_dir = tempfile.mkdtemp()
        self.db = db.EphemDB()
        self.env = Env(self.db)
        self.last_tx = None

        initial_balances = {}

        for i in range(num_accounts):
            account = accounts[i]
            initial_balances[account] = {'wei': 10 ** 24}

        for i in range(1, 5):
            address = int_to_addr(i)
            initial_balances[address] = {'wei': 1}

        self.block = blocks.genesis(
            self.env,
            start_alloc=initial_balances,
        )
        self.blocks = [self.block]
        self.block.timestamp = 1410973349
        self.block.coinbase = DEFAULT_ACCOUNT
        self.block.gas_limit = 10 ** 9

    def __del__(self):
        shutil.rmtree(self.temp_data_dir)

    def contract(self, sourcecode, sender=DEFAULT_KEY, endowment=0,  # pylint: disable=too-many-arguments
                 language='serpent', libraries=None, path=None,
                 constructor_call=None, **kwargs):
        if language not in languages:
            languages[language] = __import__(language)

        compiler = languages[language]
        bytecode = compiler.compile(sourcecode, path=path, libraries=libraries, **kwargs)

        if constructor_call is not None:
            bytecode += constructor_call

        address = self.evm(bytecode, sender, endowment)

        if len(self.block.get_code(address)) == 0:
            raise Exception('Contract code empty')

        return address

    def abi_contract(self, sourcecode, sender=DEFAULT_KEY, endowment=0,  # pylint: disable=too-many-arguments
                     language='serpent', log_listener=None, listen=True,
                     libraries=None, path=None, constructor_parameters=None,
                     **kwargs):
        # pylint: disable=too-many-locals

        compiler = languages[language]
        contract_interface = compiler.mk_full_signature(sourcecode, path=path, **kwargs)
        translator = ContractTranslator(contract_interface)

        encoded_parameters = None
        if constructor_parameters is not None:
            encoded_parameters = translator.encode_constructor_arguments(constructor_parameters)

        address = self.contract(
            sourcecode,
            sender,
            endowment,
            language,
            libraries,
            path,
            constructor_call=encoded_parameters,
            **kwargs
        )

        return ABIContract(
            self,
            translator,
            address,
            listen=listen,
            log_listener=log_listener,
        )

    def clear_listeners(self):
        while len(self.block.log_listeners):
            self.block.log_listeners.pop()

    def evm(self, code, sender=DEFAULT_KEY, endowment=0, gas=None):
        sendnonce = self.block.get_nonce(privtoaddr(sender))

        transaction = transactions.contract(sendnonce, gas_price, gas_limit, endowment, code)
        transaction.sign(sender)

        if gas is not None:
            transaction.startgas = gas

        (success, output) = processblock.apply_transaction(self.block, transaction)

        if not success:
            raise ContractCreationFailed()

        return output

    def call(*args, **kwargs):  # pylint: disable=unused-argument,no-method-argument
        raise Exception(
            'Call deprecated. Please use the abi_contract mechanism or '
            'send(sender, to, value, data) directly, using the abi module to '
            'generate data if needed.'
        )

    def _send(self, sender, to, value, evmdata='', funid=None, abi=None,  # pylint: disable=too-many-arguments
              profiling=0):
        # pylint: disable=too-many-locals

        if funid is not None or abi is not None:
            raise Exception(
                'Send with funid+abi is deprecated. Please use the abi_contract mechanism'
            )

        start_time = time.time()
        gas_used = self.block.gas_used

        sendnonce = self.block.get_nonce(privtoaddr(sender))
        transaction = transactions.Transaction(sendnonce, gas_price, gas_limit, to, value, evmdata)
        self.last_tx = transaction
        transaction.sign(sender)
        recorder = None

        if profiling > 1:
            recorder = LogRecorder(
                disable_other_handlers=True,
                log_config=TRACE_LVL_MAP[3],
            )

        try:
            (success, output) = processblock.apply_transaction(self.block, transaction)

            if not success:
                raise TransactionFailed()

            out = {
                'output': output,
            }

            if profiling > 0:
                zero_bytes_count = transaction.data.count(ascii_chr(0))
                non_zero_bytes_count = len(transaction.data) - zero_bytes_count

                zero_bytes_cost = opcodes.GTXDATAZERO * zero_bytes_count
                nonzero_bytes_cost = opcodes.GTXDATANONZERO * non_zero_bytes_count

                base_gas_cost = opcodes.GTXCOST
                intrinsic_gas_used = base_gas_cost + zero_bytes_cost + nonzero_bytes_cost

                out['time'] = time.time() - start_time
                out['gas'] = self.block.gas_used - gas_used - intrinsic_gas_used

            if profiling > 1:
                trace = recorder.pop_records()
                vm_operations = [
                    event['op']
                    for event in trace
                    if event['event'] == 'vm'
                ]
                opdict = {}
                for operation in vm_operations:
                    opdict[operation] = opdict.get(operation, 0) + 1
                out['ops'] = opdict

            return out
        finally:
            # ensure LogRecorder has been disabled
            if recorder:
                recorder.pop_records()

    def profile(self, *args, **kwargs):
        kwargs['profiling'] = True
        return self._send(*args, **kwargs)

    def send(self, *args, **kwargs):
        return self._send(*args, **kwargs)["output"]

    def mkspv(self, sender, to, value, data=None, funid=None, abi=None):  # pylint: disable=too-many-arguments
        # TODO: rewrite the method without using encode_abi/encode_datalist
        # since both function were removed.
        raise NotImplementedError()

        # if not HAS_SERPENT:
        #     raise RuntimeError('ethereum-serpent package not installed')
        # data = data or list()
        # sendnonce = self.block.get_nonce(privtoaddr(sender))
        # if funid is not None:
        #     evmdata = serpent.encode_abi(funid, *abi)
        # else:
        #     evmdata = serpent.encode_datalist(*data)
        # transaction = transactions.Transaction(sendnonce, gas_price, gas_limit,
        #                                        to, value, evmdata)
        # self.last_tx = transaction
        # transaction.sign(sender)
        # return spv.mk_transaction_spv_proof(self.block, transaction)

    def verifyspv(self, sender, to, value, data=None, funid=None, abi=None, proof=None):  # pylint: disable=too-many-arguments
        # TODO: rewrite the method without using encode_abi/encode_datalist
        # since both function were removed.
        raise NotImplementedError()

        # if not HAS_SERPENT:
        #     raise RuntimeError('ethereum-serpent package not installed')
        # data = data or list()
        # proof = proof or list()
        # sendnonce = self.block.get_nonce(privtoaddr(sender))
        # if funid is not None:
        #     evmdata = serpent.encode_abi(funid, *abi)
        # else:
        #     evmdata = serpent.encode_datalist(*data)
        # transaction = transactions.Transaction(sendnonce, gas_price,
        #                                        gas_limit, to, value, evmdata)
        # self.last_tx = transaction
        # transaction.sign(sender)
        # return spv.verify_transaction_spv_proof(self.block, transaction, proof)

    def trace(self, sender, to, value, data=None):
        # collect log events (independent of loglevel filters)
        data = data or list()
        recorder = LogRecorder()
        self.send(sender, to, value, data)
        return recorder.pop_records()

    def mine(self, number_of_blocks=1, coinbase=DEFAULT_ACCOUNT, **kwargs):
        if 'n' in kwargs:  # compatibility
            number_of_blocks = kwargs['n']
            warnings.warn(
                "The argument 'n' is deprecated and its support will be removed "
                "in the future versions. Please use the name 'number_of_blocks'."
            )

        for _ in range(number_of_blocks):
            self.block.finalize()
            self.block.commit_state()
            self.db.put(self.block.hash, rlp.encode(self.block))
            timestamp = self.block.timestamp + 6 + rand() % 12

            block = blocks.Block.init_from_parent(
                self.block,
                coinbase,
                timestamp=timestamp,
            )

            self.block = block
            self.blocks.append(self.block)

    def snapshot(self):
        return rlp.encode(self.block)

    def revert(self, data):
        self.block = rlp.decode(data, blocks.Block, env=self.env)
        # pylint: disable=protected-access
        self.block._mutable = True
        self.block.header._mutable = True
        self.block._cached_rlp = None
        self.block.header._cached_rlp = None



# -*- coding: utf-8 -*-
# ############# version ##################
from pkg_resources import get_distribution, DistributionNotFound
import os.path
import subprocess
import re
# Import slogging to patch logging as soon as possible
from . import slogging  # noqa


GIT_DESCRIBE_RE = re.compile('^(?P<version>v\d+\.\d+\.\d+)-(?P<git>\d+-g[a-fA-F0-9]+(?:-dirty)?)$')


__version__ = None
try:
    _dist = get_distribution('pyethapp')
    # Normalize case for Windows systems
    dist_loc = os.path.normcase(_dist.location)
    here = os.path.normcase(__file__)
    if not here.startswith(os.path.join(dist_loc, 'pyethapp')):
        # not installed, but there is another version that *is*
        raise DistributionNotFound
    __version__ = _dist.version
except DistributionNotFound:
    pass

if not __version__:
    try:
        rev = subprocess.check_output(['git', 'describe', '--tags', '--dirty'],
                                      stderr=subprocess.STDOUT)
        match = GIT_DESCRIBE_RE.match(rev)
        if match:
            __version__ = "{}+git-{}".format(match.group("version"), match.group("git"))
    except:
        pass

if not __version__:
    __version__ = 'undefined'

# ########### endversion ##################

'''from ethereum import utils
from ethereum import trie
from ethereum import securetrie
from ethereum import blocks
from ethereum import transactions
from ethereum import processblock
from ethereum import tester
from ethereum import abi
from ethereum import keys
from ethereum import ethash'''



import pytest
from ethereum import utils

def test_add_cool_checksum():
    addresses = [
      # All caps
      '0x52908400098527886E0F7030069857D2E4169EE7',
      '0x8617E340B3D01FA5F11F306F4090FD50E238070D',
      # All Lower
      '0xde709f2102306220921060314715629080e2fb77',
      '0x27b1fdb04752bbc536007a920d24acb045561c26',
      # Normal
      '0x5aAeb6053F3E94C9b9A09f33669435E7Ef1BeAed',
      '0xfB6916095ca1df60bB79Ce92cE3Ea74c37c5d359',
      '0xdbF03B407c01E7cD3CBea99509d93f8DDDC8C6FB',
      '0xD1220A0cf47c7B9Be7A2E6BA89F429762e7b9aDb',
    ]

    for addr in addresses:
        assert utils.add_cool_checksum(addr.lower()) == addr
        assert utils.check_and_strip_cool_checksum(addr) == utils.normalize_address(addr)



# -*- coding: utf-8 -*-
from __future__ import print_function

import ast
import re
import warnings

import yaml  # use yaml instead of json to get non unicode (works with ascii only data)
from rlp.utils import decode_hex
from ethereum.utils import encode_hex

from ethereum import utils
from ethereum.utils import (
    big_endian_to_int, ceil32, int_to_big_endian, encode_int, is_numeric, isnumeric, is_string,
    rzpad, TT255, TT256, zpad,
)

# The number of bytes is encoded as a uint256
# Type used to encode a string/bytes length
INT256 = 'uint', '256', []
lentyp = INT256  # pylint: disable=invalid-name


class EncodingError(Exception):
    pass


class ValueOutOfBounds(EncodingError):
    pass


def json_decode(data):
    return yaml.safe_load(data)


def split32(data):
    """ Split data into pieces of 32 bytes. """
    all_pieces = []

    for position in range(0, len(data), 32):
        piece = data[position:position + 32]
        all_pieces.append(piece)

    return all_pieces


def _canonical_type(name):  # pylint: disable=too-many-return-statements
    """ Replace aliases to the corresponding type to compute the ids. """

    if name == 'int':
        return 'int256'

    if name == 'uint':
        return 'uint256'

    if name == 'fixed':
        return 'fixed128x128'

    if name == 'ufixed':
        return 'ufixed128x128'

    if name.startswith('int['):
        return 'int256' + name[3:]

    if name.startswith('uint['):
        return 'uint256' + name[4:]

    if name.startswith('fixed['):
        return 'fixed128x128' + name[5:]

    if name.startswith('ufixed['):
        return 'ufixed128x128' + name[6:]

    return name


def normalize_name(name):
    """ Return normalized event/function name. """
    if '(' in name:
        return name[:name.find('(')]

    return name


def method_id(name, encode_types):
    """ Return the unique method id.

    The signature is defined as the canonical expression of the basic
    prototype, i.e. the function name with the parenthesised list of parameter
    types. Parameter types are split by a single comma - no spaces are used.

    The method id is defined as the first four bytes (left, high-order in
    big-endian) of the Keccak (SHA-3) hash of the signature of the function.
    """
    function_types = [
        _canonical_type(type_)
        for type_ in encode_types
    ]

    function_signature = '{function_name}({canonical_types})'.format(
        function_name=name,
        canonical_types=','.join(function_types),
    )

    function_keccak = utils.sha3(function_signature)
    first_bytes = function_keccak[:4]

    return big_endian_to_int(first_bytes)


def event_id(name, encode_types):
    """ Return the event id.

    Defined as:

        `keccak(EVENT_NAME+"("+EVENT_ARGS.map(canonical_type_of).join(",")+")")`

    Where `canonical_type_of` is a function that simply returns the canonical
    type of a given argument, e.g. for uint indexed foo, it would return
    uint256). Note the lack of spaces.
    """

    event_types = [
        _canonical_type(type_)
        for type_ in encode_types
    ]

    event_signature = '{event_name}({canonical_types})'.format(
        event_name=name,
        canonical_types=','.join(event_types),
    )

    return big_endian_to_int(utils.sha3(event_signature))


def decint(n, signed=False):  # pylint: disable=invalid-name,too-many-branches
    ''' Decode an unsigned/signed integer. '''

    if isinstance(n, str):
        n = utils.to_string(n)

    if n is True:
        return 1

    if n is False:
        return 0

    if n is None:
        return 0

    if is_numeric(n):
        if signed:
            if not -TT255 <= n <= TT255 - 1:
                raise EncodingError('Number out of range: %r' % n)
        else:
            if not 0 <= n <= TT256 - 1:
                raise EncodingError('Number out of range: %r' % n)

        return n

    if is_string(n):
        if len(n) > 32:
            raise EncodingError('String too long: %r' % n)

        if len(n) == 40:
            int_bigendian = decode_hex(n)
        else:
            int_bigendian = n  # pylint: disable=redefined-variable-type

        result = big_endian_to_int(int_bigendian)
        if signed:
            if result >= TT255:
                result -= TT256

            if not -TT255 <= result <= TT255 - 1:
                raise EncodingError('Number out of range: %r' % n)
        else:
            if not 0 <= result <= TT256 - 1:
                raise EncodingError('Number out of range: %r' % n)

        return result

    raise EncodingError('Cannot decode integer: %r' % n)


def encode_single(typ, arg):  # pylint: disable=too-many-return-statements,too-many-branches,too-many-statements,too-many-locals
    ''' Encode `arg` as `typ`.

    `arg` will be encoded in a best effort manner, were necessary the function
    will try to correctly define the underlying binary representation (ie.
    decoding a hex-encoded address/hash).

    Args:
        typ (Tuple[(str, int, list)]): A 3-tuple defining the `arg` type.

            The first element defines the type name.
            The second element defines the type length in bits.
            The third element defines if it's an array type.

            Together the first and second defines the elementary type, the third
            element must be present but is ignored.

            Valid type names are:
                - uint
                - int
                - bool
                - ufixed
                - fixed
                - string
                - bytes
                - hash
                - address

        arg (object): The object to be encoded, it must be a python object
            compatible with the `typ`.

    Raises:
        ValueError: when an invalid `typ` is supplied.
        ValueOutOfBounds: when `arg` cannot be encoded as `typ` because of the
            binary contraints.

    Note:
        This function don't work with array types, for that use the `enc`
        function.
    '''
    base, sub, _ = typ

    if base == 'uint':
        sub = int(sub)

        if not (0 < sub <= 256 and sub % 8 == 0):
            raise ValueError('invalid unsigned integer bit length {}'.format(sub))

        try:
            i = decint(arg, signed=False)
        except EncodingError:
            # arg is larger than 2**256
            raise ValueOutOfBounds(repr(arg))

        if not 0 <= i < 2 ** sub:
            raise ValueOutOfBounds(repr(arg))

        value_encoded = int_to_big_endian(i)
        return zpad(value_encoded, 32)

    if base == 'int':
        sub = int(sub)
        bits = sub - 1

        if not (0 < sub <= 256 and sub % 8 == 0):
            raise ValueError('invalid integer bit length {}'.format(sub))

        try:
            i = decint(arg, signed=True)
        except EncodingError:
            # arg is larger than 2**255
            raise ValueOutOfBounds(repr(arg))

        if not -2 ** bits <= i < 2 ** bits:
            raise ValueOutOfBounds(repr(arg))

        value = i % 2 ** sub  # convert negative to "equivalent" positive
        value_encoded = int_to_big_endian(value)
        return zpad(value_encoded, 32)

    if base == 'bool':
        if arg is True:
            value_encoded = int_to_big_endian(1)
        elif arg is False:
            value_encoded = int_to_big_endian(0)
        else:
            raise ValueError('%r is not bool' % arg)

        return zpad(value_encoded, 32)

    if base == 'ufixed':
        sub = str(sub)  # pylint: disable=redefined-variable-type

        high_str, low_str = sub.split('x')
        high = int(high_str)
        low = int(low_str)

        if not (0 < high + low <= 256 and high % 8 == 0 and low % 8 == 0):
            raise ValueError('invalid unsigned fixed length {}'.format(sub))

        if not 0 <= arg < 2 ** high:
            raise ValueOutOfBounds(repr(arg))

        float_point = arg * 2 ** low
        fixed_point = int(float_point)
        return zpad(int_to_big_endian(fixed_point), 32)

    if base == 'fixed':
        sub = str(sub)  # pylint: disable=redefined-variable-type

        high_str, low_str = sub.split('x')
        high = int(high_str)
        low = int(low_str)
        bits = high - 1

        if not (0 < high + low <= 256 and high % 8 == 0 and low % 8 == 0):
            raise ValueError('invalid unsigned fixed length {}'.format(sub))

        if not -2 ** bits <= arg < 2 ** bits:
            raise ValueOutOfBounds(repr(arg))

        float_point = arg * 2 ** low
        fixed_point = int(float_point)
        value = fixed_point % 2 ** 256
        return zpad(int_to_big_endian(value), 32)

    if base == 'string':
        if isinstance(arg, utils.unicode):
            arg = arg.encode('utf8')
        else:
            try:
                arg.decode('utf8')
            except UnicodeDecodeError:
                raise ValueError('string must be utf8 encoded')

        if len(sub):  # fixed length
            if not 0 <= len(arg) <= int(sub):
                raise ValueError('invalid string length {}'.format(sub))

            if not 0 <= int(sub) <= 32:
                raise ValueError('invalid string length {}'.format(sub))

            return rzpad(arg, 32)

        if not 0 <= len(arg) < TT256:
            raise Exception('Integer invalid or out of range: %r' % arg)

        length_encoded = zpad(int_to_big_endian(len(arg)), 32)
        value_encoded = rzpad(arg, utils.ceil32(len(arg)))

        return length_encoded + value_encoded

    if base == 'bytes':
        if not is_string(arg):
            raise EncodingError('Expecting string: %r' % arg)

        arg = utils.to_string(arg)  # py2: force unicode into str

        if len(sub):  # fixed length
            if not 0 <= len(arg) <= int(sub):
                raise ValueError('string must be utf8 encoded')

            if not 0 <= int(sub) <= 32:
                raise ValueError('string must be utf8 encoded')

            return rzpad(arg, 32)

        if not 0 <= len(arg) < TT256:
            raise Exception('Integer invalid or out of range: %r' % arg)

        length_encoded = zpad(int_to_big_endian(len(arg)), 32)
        value_encoded = rzpad(arg, utils.ceil32(len(arg)))

        return length_encoded + value_encoded

    if base == 'hash':
        if not (int(sub) and int(sub) <= 32):
            raise EncodingError('too long: %r' % arg)

        if isnumeric(arg):
            return zpad(encode_int(arg), 32)

        if len(arg) == int(sub):
            return zpad(arg, 32)

        if len(arg) == int(sub) * 2:
            return zpad(decode_hex(arg), 32)

        raise EncodingError('Could not parse hash: %r' % arg)

    if base == 'address':
        assert sub == ''

        if isnumeric(arg):
            return zpad(encode_int(arg), 32)

        if len(arg) == 20:
            return zpad(arg, 32)

        if len(arg) == 40:
            return zpad(decode_hex(arg), 32)

        if len(arg) == 42 and arg[:2] == '0x':
            return zpad(decode_hex(arg[2:]), 32)

        raise EncodingError('Could not parse address: %r' % arg)
    raise EncodingError('Unhandled type: %r %r' % (base, sub))


class ContractTranslator(object):

    def __init__(self, contract_interface):
        if is_string(contract_interface):
            contract_interface = json_decode(contract_interface)

        self.fallback_data = None
        self.constructor_data = None
        self.function_data = {}
        self.event_data = {}

        for description in contract_interface:
            entry_type = description.get('type', 'function')
            encode_types = []
            signature = []

            # If it's a function/constructor/event
            if entry_type != 'fallback' and 'inputs' in description:
                encode_types = [
                    element['type']
                    for element in description.get('inputs')
                ]

                signature = [
                    (element['type'], element['name'])
                    for element in description.get('inputs')
                ]

            if entry_type == 'function':
                normalized_name = normalize_name(description['name'])

                decode_types = [
                    element['type']
                    for element in description['outputs']
                ]

                self.function_data[normalized_name] = {
                    'prefix': method_id(normalized_name, encode_types),
                    'encode_types': encode_types,
                    'decode_types': decode_types,
                    'is_constant': description.get('constant', False),
                    'signature': signature,
                    'payable': description.get('payable', False),
                }

            elif entry_type == 'event':
                normalized_name = normalize_name(description['name'])

                indexed = [
                    element['indexed']
                    for element in description['inputs']
                ]
                names = [
                    element['name']
                    for element in description['inputs']
                ]
                # event_id == topics[0]
                self.event_data[event_id(normalized_name, encode_types)] = {
                    'types': encode_types,
                    'name': normalized_name,
                    'names': names,
                    'indexed': indexed,
                    'anonymous': description.get('anonymous', False),
                }

            elif entry_type == 'constructor':
                if self.constructor_data is not None:
                    raise ValueError('Only one constructor is supported.')

                self.constructor_data = {
                    'encode_types': encode_types,
                    'signature': signature,
                }

            elif entry_type == 'fallback':
                if self.fallback_data is not None:
                    raise ValueError('Only one fallback function is supported.')
                self.fallback_data = {'payable': description['payable']}

            else:
                raise ValueError('Unknown type {}'.format(description['type']))

    def encode(self, function_name, args):
        warnings.warn('encode is deprecated, please use encode_function_call', DeprecationWarning)
        return self.encode_function_call(function_name, args)

    def decode(self, function_name, data):
        warnings.warn('decode is deprecated, please use decode_function_result', DeprecationWarning)
        return self.decode_function_result(function_name, data)

    def encode_function_call(self, function_name, args):
        """ Return the encoded function call.

        Args:
            function_name (str): One of the existing functions described in the
                contract interface.
            args (List[object]): The function arguments that wll be encoded and
                used in the contract execution in the vm.

        Return:
            bin: The encoded function name and arguments so that it can be used
                 with the evm to execute a funcion call, the binary string follows
                 the Ethereum Contract ABI.
        """
        if function_name not in self.function_data:
            raise ValueError('Unkown function {}'.format(function_name))

        description = self.function_data[function_name]

        function_selector = zpad(encode_int(description['prefix']), 4)
        arguments = encode_abi(description['encode_types'], args)

        return function_selector + arguments

    def decode_function_result(self, function_name, data):
        """ Return the function call result decoded.

        Args:
            function_name (str): One of the existing functions described in the
                contract interface.
            data (bin): The encoded result from calling `function_name`.

        Return:
            List[object]: The values returned by the call to `function_name`.
        """
        description = self.function_data[function_name]
        arguments = decode_abi(description['decode_types'], data)
        return arguments

    def encode_constructor_arguments(self, args):
        """ Return the encoded constructor call. """
        if self.constructor_data is None:
            raise ValueError("The contract interface didn't have a constructor")

        return encode_abi(self.constructor_data['encode_types'], args)

    def decode_event(self, log_topics, log_data):
        """ Return a dictionary representation the log.

        Note:
            This function won't work with anonymous events.

        Args:
            log_topics (List[bin]): The log's indexed arguments.
            log_data (bin): The encoded non-indexed arguments.
        """
        # https://github.com/ethereum/wiki/wiki/Ethereum-Contract-ABI#function-selector-and-argument-encoding

        # topics[0]: keccak(EVENT_NAME+"("+EVENT_ARGS.map(canonical_type_of).join(",")+")")
        # If the event is declared as anonymous the topics[0] is not generated;
        if not len(log_topics) or log_topics[0] not in self.event_data:
            raise ValueError('Unknown log type')

        event_id_ = log_topics[0]

        event = self.event_data[event_id_]

        # data: abi_serialise(EVENT_NON_INDEXED_ARGS)
        # EVENT_NON_INDEXED_ARGS is the series of EVENT_ARGS that are not
        # indexed, abi_serialise is the ABI serialisation function used for
        # returning a series of typed values from a function.
        unindexed_types = [
            type_
            for type_, indexed in zip(event['types'], event['indexed'])
            if not indexed
        ]
        unindexed_args = decode_abi(unindexed_types, log_data)

        # topics[n]: EVENT_INDEXED_ARGS[n - 1]
        # EVENT_INDEXED_ARGS is the series of EVENT_ARGS that are indexed
        indexed_count = 1  # skip topics[0]

        result = {}
        for name, type_, indexed in zip(event['names'], event['types'], event['indexed']):
            if indexed:
                topic_bytes = utils.zpad(
                    utils.encode_int(log_topics[indexed_count]),
                    32,
                )
                indexed_count += 1
                value = decode_single(process_type(type_), topic_bytes)
            else:
                value = unindexed_args.pop(0)

            result[name] = value
        result['_event_type'] = utils.to_string(event['name'])

        return result

    def listen(self, log, noprint=True):
        """
        Return a dictionary representation of the Log instance.

        Note:
            This function won't work with anonymous events.

        Args:
            log (processblock.Log): The Log instance that needs to be parsed.
            noprint (bool): Flag to turn off priting of the decoded log instance.
        """
        try:
            result = self.decode_event(log.topics, log.data)
        except ValueError:
            return  # api compatibility

        if not noprint:
            print(result)

        return result


def process_type(typ):
    # Crazy reg expression to separate out base type component (eg. uint),
    # size (eg. 256, 128x128, none), array component (eg. [], [45], none)
    regexp = '([a-z]*)([0-9]*x?[0-9]*)((\[[0-9]*\])*)'
    base, sub, arr, _ = re.match(regexp, utils.to_string_for_regexp(typ)).groups()
    arrlist = re.findall('\[[0-9]*\]', arr)
    assert len(''.join(arrlist)) == len(arr), \
        "Unknown characters found in array declaration"
    # Check validity of string type
    if base == 'string' or base == 'bytes':
        assert re.match('^[0-9]*$', sub), \
            "String type must have no suffix or numerical suffix"
        assert not sub or int(sub) <= 32, \
            "Maximum 32 bytes for fixed-length str or bytes"
    # Check validity of integer type
    elif base == 'uint' or base == 'int':
        assert re.match('^[0-9]+$', sub), \
            "Integer type must have numerical suffix"
        assert 8 <= int(sub) <= 256, \
            "Integer size out of bounds"
        assert int(sub) % 8 == 0, \
            "Integer size must be multiple of 8"
    # Check validity of fixed type
    elif base == 'ufixed' or base == 'fixed':
        assert re.match('^[0-9]+x[0-9]+$', sub), \
            "Real type must have suffix of form <high>x<low>, eg. 128x128"
        high, low = [int(x) for x in sub.split('x')]
        assert 8 <= (high + low) <= 256, \
            "Real size out of bounds (max 32 bytes)"
        assert high % 8 == 0 and low % 8 == 0, \
            "Real high/low sizes must be multiples of 8"
    # Check validity of hash type
    elif base == 'hash':
        assert re.match('^[0-9]+$', sub), \
            "Hash type must have numerical suffix"
    # Check validity of address type
    elif base == 'address':
        assert sub == '', "Address cannot have suffix"
    return base, sub, [ast.literal_eval(x) for x in arrlist]


# Returns the static size of a type, or None if dynamic
def get_size(typ):
    base, sub, arrlist = typ
    if not len(arrlist):
        if base in ('string', 'bytes') and not sub:
            return None
        return 32
    if arrlist[-1] == []:
        return None
    o = get_size((base, sub, arrlist[:-1]))
    if o is None:
        return None
    return arrlist[-1][0] * o


# Encodes a single value (static or dynamic)
def enc(typ, arg):
    base, sub, arrlist = typ
    type_size = get_size(typ)

    if base in ('string', 'bytes') and not sub:
        return encode_single(typ, arg)

    # Encode dynamic-sized lists via the head/tail mechanism described in
    # https://github.com/ethereum/wiki/wiki/Proposal-for-new-ABI-value-encoding
    if type_size is None:
        assert isinstance(arg, list), \
            "Expecting a list argument"
        subtyp = base, sub, arrlist[:-1]
        subsize = get_size(subtyp)
        myhead, mytail = b'', b''
        if arrlist[-1] == []:
            myhead += enc(INT256, len(arg))
        else:
            assert len(arg) == arrlist[-1][0], \
                "Wrong array size: found %d, expecting %d" % \
                (len(arg), arrlist[-1][0])
        for i in range(len(arg)):
            if subsize is None:
                myhead += enc(INT256, 32 * len(arg) + len(mytail))
                mytail += enc(subtyp, arg[i])
            else:
                myhead += enc(subtyp, arg[i])
        return myhead + mytail
    # Encode static-sized lists via sequential packing
    else:
        if arrlist == []:
            return utils.to_string(encode_single(typ, arg))
        else:
            subtyp = base, sub, arrlist[:-1]
            o = b''
            for x in arg:
                o += enc(subtyp, x)
            return o


# Encodes multiple arguments using the head/tail mechanism
def encode_abi(types, args):
    headsize = 0
    proctypes = [process_type(typ) for typ in types]
    sizes = [get_size(typ) for typ in proctypes]
    for i, arg in enumerate(args):
        if sizes[i] is None:
            headsize += 32
        else:
            headsize += sizes[i]
    myhead, mytail = b'', b''
    for i, arg in enumerate(args):
        if sizes[i] is None:
            myhead += enc(INT256, headsize + len(mytail))
            mytail += enc(proctypes[i], args[i])
        else:
            myhead += enc(proctypes[i], args[i])
    return myhead + mytail


# Decodes a single base datum
def decode_single(typ, data):
    base, sub, _ = typ
    if base == 'address':
        return encode_hex(data[12:])
    elif base == 'hash':
        return data[32 - int(sub):]
    elif base == 'string' or base == 'bytes':
        if len(sub):
            return data[:int(sub)]
        else:
            l = big_endian_to_int(data[0:32])
            return data[32:][:l]
    elif base == 'uint':
        return big_endian_to_int(data)
    elif base == 'int':
        o = big_endian_to_int(data)
        return (o - 2 ** int(sub)) if o >= 2 ** (int(sub) - 1) else o
    elif base == 'ufixed':
        high, low = [int(x) for x in sub.split('x')]
        return big_endian_to_int(data) * 1.0 // 2 ** low
    elif base == 'fixed':
        high, low = [int(x) for x in sub.split('x')]
        o = big_endian_to_int(data)
        i = (o - 2 ** (high + low)) if o >= 2 ** (high + low - 1) else o
        return (i * 1.0 // 2 ** low)
    elif base == 'bool':
        return bool(int(encode_hex(data), 16))


# Decodes multiple arguments using the head/tail mechanism
def decode_abi(types, data):
    # Process types
    proctypes = [process_type(typ) for typ in types]
    # Get sizes of everything
    sizes = [get_size(typ) for typ in proctypes]
    # Initialize array of outputs
    outs = [None] * len(types)
    # Initialize array of start positions
    start_positions = [None] * len(types) + [len(data)]
    # If a type is static, grab the data directly, otherwise record
    # its start position
    pos = 0
    for i, typ in enumerate(types):
        if sizes[i] is None:
            start_positions[i] = big_endian_to_int(data[pos:pos + 32])
            j = i - 1
            while j >= 0 and start_positions[j] is None:
                start_positions[j] = start_positions[i]
                j -= 1
            pos += 32
        else:
            outs[i] = data[pos:pos + sizes[i]]
            pos += sizes[i]
    # We add a start position equal to the length of the entire data
    # for convenience.
    j = len(types) - 1
    while j >= 0 and start_positions[j] is None:
        start_positions[j] = start_positions[len(types)]
        j -= 1
    assert pos <= len(data), "Not enough data for head"
    # Grab the data for tail arguments using the start positions
    # calculated above
    for i, typ in enumerate(types):
        if sizes[i] is None:
            offset = start_positions[i]
            next_offset = start_positions[i + 1]
            outs[i] = data[offset:next_offset]
    # Recursively decode them all
    return [dec(proctypes[i], outs[i]) for i in range(len(outs))]


# Decode a single value (static or dynamic)
def dec(typ, arg):
    base, sub, arrlist = typ
    sz = get_size(typ)
    # Dynamic-sized strings are encoded as <len(str)> + <str>
    if base in ('string', 'bytes') and not sub:
        L = big_endian_to_int(arg[:32])
        assert len(arg[32:]) == ceil32(L), "Wrong data size for string/bytes object"
        return arg[32:][:L]
    # Dynamic-sized arrays
    elif sz is None:
        L = big_endian_to_int(arg[:32])
        subtyp = base, sub, arrlist[:-1]
        subsize = get_size(subtyp)
        # If children are dynamic, use the head/tail mechanism. Fortunately,
        # here the code is simpler since we do not have to worry about
        # mixed dynamic and static children, as we do in the top-level multi-arg
        # case
        if subsize is None:
            assert len(arg) >= 32 + 32 * L, "Not enough data for head"
            start_positions = [big_endian_to_int(arg[32 + 32 * i: 64 + 32 * i])
                               for i in range(L)] + [len(arg)]
            outs = [arg[start_positions[i]: start_positions[i + 1]]
                    for i in range(L)]
            return [dec(subtyp, out) for out in outs]
        # If children are static, then grab the data slice for each one and
        # sequentially decode them manually
        else:
            return [dec(subtyp, arg[32 + subsize * i: 32 + subsize * (i + 1)])
                    for i in range(L)]
    # Static-sized arrays: decode piece-by-piece
    elif len(arrlist):
        L = arrlist[-1][0]
        subtyp = base, sub, arrlist[:-1]
        subsize = get_size(subtyp)
        return [dec(subtyp, arg[subsize * i:subsize * (i + 1)])
                for i in range(L)]
    else:
        return decode_single(typ, arg)



from ethereum import opcodes

opcode_gas = {
    opcode: gas for (opcode, ins, outs, gas) in opcodes.opcodes.values()
}


def test_eip150_opcode_gascost():
    """Ensure gas prices specified in
    https://github.com/ethereum/eips/issues/150
    """
    assert opcode_gas['EXTCODESIZE'] + opcodes.EXTCODELOAD_SUPPLEMENTAL_GAS == 700
    assert opcode_gas['EXTCODECOPY'] + opcodes.EXTCODELOAD_SUPPLEMENTAL_GAS == 700
    assert opcode_gas['BALANCE'] + opcodes.BALANCE_SUPPLEMENTAL_GAS == 400
    assert opcode_gas['SLOAD'] + opcodes.SLOAD_SUPPLEMENTAL_GAS == 200

    assert opcode_gas['CALL'] + opcodes.CALL_SUPPLEMENTAL_GAS == 700
    assert opcode_gas['DELEGATECALL'] + opcodes.CALL_SUPPLEMENTAL_GAS == 700
    assert opcode_gas['CALLCODE'] + opcodes.CALL_SUPPLEMENTAL_GAS == 700

    assert opcode_gas['SUICIDE'] + opcodes.SUICIDE_SUPPLEMENTAL_GAS == 5000



# -*- coding: utf-8 -*-
from os import path

import pytest

from ethereum.utils import encode_hex

from ethereum import tester
from ethereum import utils
from ethereum import _solidity
from ethereum._solidity import get_solidity

SOLIDITY_AVAILABLE = get_solidity() is not None
CONTRACTS_DIR = path.join(path.dirname(__file__), 'contracts')


def bytecode_is_generated(cinfo, cname):
    return 'code' in cinfo[cname] and len(cinfo[cname]['code']) > 10


@pytest.mark.skipif(not SOLIDITY_AVAILABLE, reason='solc compiler not available')
def test_library_from_file():
    state = tester.state()
    state.env.config['HOMESTEAD_FORK_BLKNUM'] = 0  # enable CALLCODE opcode

    library = state.abi_contract(
        None,
        path=path.join(CONTRACTS_DIR, 'seven_library.sol'),
        language='solidity',
    )

    libraries = {
        'SevenLibrary': encode_hex(library.address),
    }
    contract = state.abi_contract(
        None,
        path=path.join(CONTRACTS_DIR, 'seven_contract.sol'),
        libraries=libraries,
        language='solidity',
    )

    # pylint: disable=no-member
    assert library.seven() == 7
    assert contract.test() == 7


@pytest.mark.skipif(not SOLIDITY_AVAILABLE, reason='solc compiler not available')
def test_library_from_code():
    with open(path.join(CONTRACTS_DIR, 'seven_library.sol')) as handler:
        library_code = handler.read()

    with open(path.join(CONTRACTS_DIR, 'seven_contract_without_import.sol')) as handler:
        contract_code = handler.read()

    state = tester.state()
    state.env.config['HOMESTEAD_FORK_BLKNUM'] = 0  # enable CALLCODE opcode

    library = state.abi_contract(
        library_code,
        path=None,
        language='solidity',
    )

    libraries = {
        'SevenLibrary': encode_hex(library.address),
    }
    contract = state.abi_contract(
        contract_code,
        path=None,
        libraries=libraries,
        language='solidity',
    )

    # pylint: disable=no-member
    assert library.seven() == 7
    assert contract.test() == 7


def test_names():
    with open(path.join(CONTRACTS_DIR, 'contract_names.sol')) as handler:
        code = handler.read()

    names_in_order = _solidity.solidity_names(code)

    assert ('library', 'InComment') not in names_in_order
    assert ('contract', 'InComment') not in names_in_order

    assert ('contract', 'WithSpace') in names_in_order
    assert ('contract', 'WithLineBreak') in names_in_order

    assert names_in_order == [
        ('contract', 'AContract'),
        ('library', 'ALibrary'),
        ('contract', 'WithSpace'),
        ('contract', 'WithLineBreak'),
    ]


def test_symbols():
    assert _solidity.solidity_library_symbol('a') == '__a_____________________________________'
    assert _solidity.solidity_library_symbol('aaa') == '__aaa___________________________________'
    assert _solidity.solidity_library_symbol('a' * 40) == '__aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa__'

    # the address should be sanitized when it's given to the function
    with pytest.raises(Exception):
        _solidity.solidity_resolve_address(
            'beef__a_____________________________________cafe',
            '__a_____________________________________',
            '0x1111111111111111111111111111111111111111'
        )

    # the address needs to be hex encoded
    with pytest.raises(Exception):
        _solidity.solidity_resolve_address(
            'beef__a_____________________________________cafe',
            '__a_____________________________________',
            '111111111111111111111111111111111111111_'
        )

    assert _solidity.solidity_resolve_address(
        'beef__a_____________________________________cafe',
        '__a_____________________________________',
        '1111111111111111111111111111111111111111'
    ) == 'beef1111111111111111111111111111111111111111cafe'


@pytest.mark.skipif(not SOLIDITY_AVAILABLE, reason='solc compiler not available')
def test_interop():
    serpent_contract = """\
extern solidity: [sub2:[]:i]

def main(a):
    return(a.sub2() * 2)

def sub1():
    return(5)

"""

    solidity_contract = """
    contract serpent { function sub1() returns (int256 y) {} }

    contract zoo {
        function main(address a) returns (int256 y) {
            y = serpent(a).sub1() * 2;
        }
        function sub2() returns (int256 y) {
            y = 7;
        }
        function sub3(address a) returns (address b) {
            b = a;
        }
    }
    """

    state = tester.state()
    serpent_abi = state.abi_contract(serpent_contract)
    solidity_abi = state.abi_contract(solidity_contract, language='solidity')  # should be zoo
    solidity_address = utils.encode_hex(solidity_abi.address)

    # pylint: disable=no-member
    assert serpent_abi.sub1() == 5
    assert serpent_abi.main(solidity_abi.address) == 14

    assert solidity_abi.sub2() == 7
    assert solidity_abi.sub3(utils.encode_hex(solidity_abi.address)) == solidity_address
    assert solidity_abi.main(serpent_abi.address) == 10


@pytest.mark.skipif(not SOLIDITY_AVAILABLE, reason='solc compiler not available')
def test_constructor():
    constructor_contract = '''
    contract testme {
        uint value;
        function testme(uint a) {
            value = a;
        }
        function getValue() returns (uint) {
            return value;
        }
    }
    '''

    state = tester.state()
    contract = state.abi_contract(
        constructor_contract,
        language='solidity',
        constructor_parameters=(2, ),
    )

    # pylint: disable=no-member
    assert contract.getValue() == 2


@pytest.mark.skipif(not SOLIDITY_AVAILABLE, reason='solc compiler not available')
def test_solidity_compile_rich():
    compile_rich_contract = """
    contract contract_add {
        function add7(uint a) returns(uint d) { return a + 7; }
        function add42(uint a) returns(uint d) { return a + 42; }
    }
    contract contract_sub {
        function subtract7(uint a) returns(uint d) { return a - 7; }
        function subtract42(uint a) returns(uint d) { return a - 42; }
    }
    """

    contract_info = get_solidity().compile_rich(compile_rich_contract)

    assert len(contract_info) == 2
    assert set(contract_info.keys()) == {'contract_add', 'contract_sub'}
    assert set(contract_info['contract_add'].keys()) == {'info', 'code'}
    assert set(contract_info['contract_add']['info'].keys()) == {
        'language', 'languageVersion', 'abiDefinition', 'source',
        'compilerVersion', 'developerDoc', 'userDoc'
    }
    assert bytecode_is_generated(contract_info, 'contract_add')
    assert bytecode_is_generated(contract_info, 'contract_sub')

    assert {
        defn['name']
        for defn
        in contract_info['contract_add']['info']['abiDefinition']
    } == {'add7', 'add42'}
    assert {
        defn['name']
        for defn
        in contract_info['contract_sub']['info']['abiDefinition']
    } == {'subtract7', 'subtract42'}


@pytest.mark.skipif(not SOLIDITY_AVAILABLE, reason='solc compiler not available')
def test_abi_contract():
    one_contract = """
    contract foo {
        function seven() returns (int256 y) {
            y = 7;
        }
        function mul2(int256 x) returns (int256 y) {
            y = x * 2;
        }
    }
    """

    state = tester.state()
    contract = state.abi_contract(one_contract, language='solidity')

    # pylint: disable=no-member
    assert contract.seven() == 7
    assert contract.mul2(2) == 4
    assert contract.mul2(-2) == -4


@pytest.mark.skipif(not SOLIDITY_AVAILABLE, reason='solc compiler not available')
def test_extra_args():
    src = """
    contract foo {
        function add7(uint a) returns(uint d) { return a + 7; }
        function add42(uint a) returns(uint d) { return a + 42; }
    }
    """

    contract_info = get_solidity().compile_rich(
        src,
        extra_args="--optimize-runs 100"
    )
    assert bytecode_is_generated(contract_info, 'foo')

    contract_info = get_solidity().compile_rich(
        src,
        extra_args=["--optimize-runs", "100"]
    )
    assert bytecode_is_generated(contract_info, 'foo')

def test_missing_solc(monkeypatch):
    monkeypatch.setattr(_solidity, 'get_compiler_path', lambda: None)
    assert _solidity.get_compiler_path() is None
    sample_sol_code = "contract SampleContract {}"
    with pytest.raises(_solidity.SolcMissing):
        _solidity.compile_code(sample_sol_code)



from ethereum import blocks, utils, db
from ethereum.exceptions import VerificationFailed, InvalidTransaction
import rlp
from rlp.utils import decode_hex,  str_to_bytes
from ethereum.utils import encode_hex
from rlp import DecodingError, DeserializationError
import sys
import ethereum.testutils as testutils
from ethereum.testutils import get_config_overrides
import copy

from ethereum.slogging import get_logger
logger = get_logger()


def translate_keys(olddict, keymap, valueconv, deletes):
    o = {}
    for k in list(olddict.keys()):
        if k not in deletes:
            o[keymap.get(k, k)] = valueconv(k, olddict[k])
    return o

env = blocks.Env(db._EphemDB())

translator_list = {
    "extra_data": "extraData",
    "gas_limit": "gasLimit",
    "gas_used": "gasUsed",
    "mixhash": "mixHash",
    "prevhash": "parentHash",
    "receipts_root": "receiptTrie",
    "tx_list_root": "transactionsTrie",
    "uncles_hash": "uncleHash",
    "gas_price": "gasPrice",
    "header": "blockHeader",
    "uncles": "uncleHeaders"
}


def valueconv(k, v):
    if k in ['r', 's']:
        return '0x' + encode_hex(utils.int_to_big_endian(v))
    return v


def run_block_test(params, config_overrides={}):
    b = blocks.genesis(env, start_alloc=params["pre"])
    gbh = params["genesisBlockHeader"]
    b.bloom = utils.scanners['int256b'](gbh["bloom"])
    b.timestamp = utils.scanners['int'](gbh["timestamp"])
    b.nonce = utils.scanners['bin'](gbh["nonce"])
    b.extra_data = utils.scanners['bin'](gbh["extraData"])
    b.gas_limit = utils.scanners['int'](gbh["gasLimit"])
    b.gas_used = utils.scanners['int'](gbh["gasUsed"])
    b.coinbase = utils.scanners['addr'](decode_hex(gbh["coinbase"]))
    b.difficulty = utils.parse_int_or_hex(gbh["difficulty"])
    b.prevhash = utils.scanners['bin'](gbh["parentHash"])
    b.mixhash = utils.scanners['bin'](gbh["mixHash"])
    assert b.receipts.root_hash == \
        utils.scanners['bin'](gbh["receiptTrie"])
    assert b.transactions.root_hash == \
        utils.scanners['bin'](gbh["transactionsTrie"])
    assert utils.sha3rlp(b.uncles) == \
        utils.scanners['bin'](gbh["uncleHash"])
    h = encode_hex(b.state.root_hash)
    if h != str_to_bytes(gbh["stateRoot"]):
        raise Exception("state root mismatch")
    if b.hash != utils.scanners['bin'](gbh["hash"]):
        raise Exception("header hash mismatch")
    # assert b.header.check_pow()
    blockmap = {b.hash: b}
    env.db.put(b.hash, rlp.encode(b))
    old_config = copy.deepcopy(env.config)
    try:
        for k, v in config_overrides.items():
            env.config[k] = v
        b2 = None
        for blk in params["blocks"]:
            if 'blockHeader' not in blk:
                try:
                    rlpdata = decode_hex(blk["rlp"][2:])
                    blkparent = rlp.decode(
                        rlp.encode(rlp.decode(rlpdata)[0]), blocks.BlockHeader).prevhash
                    b2 = rlp.decode(rlpdata, blocks.Block, parent=blockmap[blkparent], env=env)
                    success = b2.validate_uncles()
                except (ValueError, TypeError, AttributeError, VerificationFailed,
                        DecodingError, DeserializationError, InvalidTransaction, KeyError):
                    success = False
                assert not success
            else:
                rlpdata = decode_hex(blk["rlp"][2:])
                blkparent = rlp.decode(rlp.encode(rlp.decode(rlpdata)[0]), blocks.BlockHeader).prevhash
                b2 = rlp.decode(rlpdata, blocks.Block, parent=blockmap[blkparent], env=env)
                assert b2.validate_uncles()
                blockmap[b2.hash] = b2
                env.db.put(b2.hash, rlp.encode(b2))
            if b2:
                print('Block %d with state root %s' % (b2.number, encode_hex(b2.state.root_hash)))
            # blkdict = b.to_dict(False, True, False, True)
            # assert blk["blockHeader"] == \
            #     translate_keys(blkdict["header"], translator_list, lambda y, x: x, [])
            # assert blk["transactions"] == \
            #     [translate_keys(t, translator_list, valueconv, ['hash'])
            #      for t in blkdict["transactions"]]
            # assert blk["uncleHeader"] == \
            #     [translate_keys(u, translator_list, lambda x: x, [])
            #      for u in blkdict["uncles"]]
    finally:
        env.config = old_config


def test_block(filename, testname, testdata):
    config_overrides = get_config_overrides(filename)
    run_block_test(testdata, config_overrides=config_overrides)


excludes = {
    ('bcWalletTest.json', u'walletReorganizeOwners'),
    ('bl10251623GO.json', u'randomBlockTest'),
    ('bl201507071825GO.json', u'randomBlockTest')
}


def pytest_generate_tests(metafunc):
    testutils.generate_test_params(
        'BlockchainTests',
        metafunc,
        lambda filename, testname, _: (filename.split('/')[-1], testname) in excludes
    )


def main():
    assert len(sys.argv) >= 2, "Please specify file or dir name"
    fixtures = testutils.get_tests_from_file_or_dir(sys.argv[1])
    if len(sys.argv) >= 3:
        for filename, tests in list(fixtures.items()):
            for testname, testdata in list(tests.items()):
                if testname == sys.argv[2]:
                    print("Testing: %s %s" % (filename, testname))
                    config_overrides = get_config_overrides(filename)
                    run_block_test(testdata, config_overrides=config_overrides)
    else:
        for filename, tests in list(fixtures.items()):
            for testname, testdata in list(tests.items()):
                if (filename.split('/')[-1], testname) not in excludes:
                    print("Testing: %s %s" % (filename, testname))
                    config_overrides = get_config_overrides(filename)
                    run_block_test(testdata, config_overrides=config_overrides)


if __name__ == '__main__':
    main()



from setuptools import setup, find_packages


with open('README.rst') as readme_file:
    readme = readme_file.read()

# requirements
install_requires = set(x.strip() for x in open('requirements.txt'))
install_requires_replacements = {
    'https://github.com/ethereum/ethash/tarball/master': 'pyethash',
}
install_requires = [install_requires_replacements.get(r, r) for r in install_requires]

# dev requirements
tests_require = set(x.strip() for x in open('dev_requirements.txt'))
tests_require_replacements = dict()
tests_require_replacements = {
    'https://github.com/ethereum/serpent/tarball/develop': 'ethereum-serpent>=2.0.2'
}
tests_require = [tests_require_replacements.get(r, r) for r in tests_require]

# *IMPORTANT*: Don't manually change the version here. Use the 'bumpversion' utility.
# see: https://github.com/ethereum/pyethapp/wiki/Development:-Versions-and-Releases
version = '1.6.1'

setup(
    name="ethereum",
    packages=find_packages("."),
    description='Next generation cryptocurrency network',
    long_description=readme,
    url='https://github.com/ethereum/pyethereum/',
    install_requires=install_requires,
    tests_require=tests_require,
    setup_requires=[
        'pytest-runner==2.7'
    ],
    version=version,
    classifiers=[
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)



# -*- coding: utf8 -*-
import json
from os import path

import pytest

from ethereum.tester import state, ABIContract
from ethereum._solidity import (
        get_solidity,
        compile_file,
        solidity_get_contract_data,
        )

SOLIDITY_AVAILABLE = get_solidity() is not None
CONTRACTS_DIR = path.join(path.dirname(__file__), 'contracts')


@pytest.mark.skipif(not SOLIDITY_AVAILABLE, reason='solc compiler not available')
def test_abicontract_interface():
    """ Test for issue #370. """
    tester_state = state()

    contract_path = path.join(CONTRACTS_DIR, 'simple_contract.sol')
    contract_name = 'Simple'
    simple_compiled = compile_file(contract_path)
    simple_data = solidity_get_contract_data(
            simple_compiled,
            contract_path,
            contract_name,
            )
    simple_address = tester_state.evm(simple_data['bin'])

    # ABIContract class must accept json_abi
    abi_json = json.dumps(simple_data['abi']).encode('utf-8')

    abi = ABIContract(
        _state=tester_state,
        _abi=abi_json,
        address=simple_address,
        listen=False,
        log_listener=None,
        default_key=None,
    )

    assert abi.test() == 1  # pylint: disable=no-member
